package com.gmei.up.utils

import scala.collection.mutable.ListBuffer
import scala.util.Try
// import com.sksamuel.elastic4s.http.{ JavaClient, HttpClient }

import org.apache.http.HttpHost
import org.apache.http.auth.{ AuthScope, UsernamePasswordCredentials }
import org.apache.http.impl.client.BasicCredentialsProvider
import org.apache.http.impl.nio.client.HttpAsyncClientBuilder
import org.elasticsearch.client.RestClient
import org.elasticsearch.client.RestClientBuilder.HttpClientConfigCallback
import com.sksamuel.elastic4s.http.ElasticDsl._
import com.sksamuel.elastic4s.http.ElasticClient
// import com.sksamuel.elastic4s.requests.searches.{ SearchRequest, SearchResponse }
import com.sksamuel.elastic4s.{ HitReader, Hit }
import org.json4s.JsonDSL._
import org.json4s.jackson.JsonMethods._
import org.json4s.JObject
// import scala.concurrent._
// import scala.concurrent.duration._
// import ExecutionContext.Implicits.global

object ESClient {
  private class ElasticCredentials(username: String, password: String) extends HttpClientConfigCallback {
    val credentials = new UsernamePasswordCredentials(username, password)
    val credentialsProvider = new BasicCredentialsProvider()
    credentialsProvider.setCredentials(AuthScope.ANY, credentials)

    override def customizeHttpClient(httpClientBuilder: HttpAsyncClientBuilder): HttpAsyncClientBuilder =
      httpClientBuilder.setDefaultCredentialsProvider(credentialsProvider)
  }

  // ElasticClient
  def create(hostname: String, port: Int, username: String, password: String): ElasticClient = {
    val restClient = RestClient
      .builder(new HttpHost(hostname, port, "http"))
      .setHttpClientConfigCallback(new ElasticCredentials(username, password))
      .build()

    ElasticClient.fromRestClient(restClient)
    // ElasticClient(JavaClient.fromRestClient(restClient))
  }
}

object ES {
  case class Character(id: String, index: String, projects: String)

  implicit object CharacterHitReader extends HitReader[Character] {
    override def read(hit: Hit): Try[Character] = {
      val source = hit.sourceAsMap
      Try(Character(source("id").toString, hit.index, source("tags_v3").toString))
    }
  }

  def test(diaryQuery: String, tractateQuery: String) = {
    // TODO read from config
    val client: ElasticClient = ESClient.create("172.16.52.33", 9200, "elastic", "gengmei!@#")

    // val bb = multi(
    //   search("gm-dbmw-diary-read").source(s"""${diaryQuery}"""),
    //   search("gm-dbmw-tractate-read").source(tractateQuery)
    // )

    // println(bb.show)
    // println(diaryQuery)
    // println(tractateQuery)

    // // TODO remove await
    // val resp = client.execute(bb).await

    // // println(resp)

    // val a = resp.result.to[Character]

    // println("%%%%%%%%%%%")
    // println(a.size)
    // a.foreach { x =>
    //   println(x.id, x.index, x.projects)
    // }
    // println("%%%%%%%%%%%")

    // resp

    val a = """
{
  "query": {
    "bool": {
      "must": [
        { "term": { "is_online": true } },
        { "range": { "content_level": { "gte": 3 } } }
      ]
    }
  }
}
"""

    val resp = client.execute {
      search("gm-dbmw-diary-read").source(a)

    }.await

    println(resp)

    resp
  }

  // SearchRequest
  def generateDiaryQuery(projects: List[String], secondDemands: List[String], cityId: Int = -1): String = {
    val size = 30
    val includes = List("id", "tags_v3")

    // TODO read

    val functions = new ListBuffer[JObject]()
    val shoulds = new ListBuffer[JObject]()
    if (projects.size > 0) {
      val promotes: List[JObject] =
        List(("terms" -> ("tags_v3" -> projects)), ("term" -> ("is_promote" -> true)))

      functions += ("filter" -> ("bool" -> ("must" -> promotes))) ~ ("weight" -> 60)

      functions += ("filter" -> ("terms" -> { "tags_v3" -> projects })) ~ ("weight" -> 50)

      shoulds += ("terms" -> ("tags_v3" -> projects))
    }

    if (secondDemands.size > 0) {
      functions += ("filter" -> ("terms" -> ("second_demands" -> secondDemands))) ~ ("weight" -> 40)

      shoulds += ("terms" -> ("second_demands" -> secondDemands))
    }

    val query = s"""
{
  "query": {
    "function_score": {
      "functions": ${pretty(render(functions.toList))},
      "boost_mode": "replace",
      "score_mode": "max",
      "query": {
        "bool": {
          "should": ${pretty(render(shoulds.toList))},
          "minimum_should_match": 1,
          "must": [
            {
              "term": {
                "is_online": "true"
              }
            },
            {
              "term": {
                "has_cover": "true"
              }
            },
            {
              "term": {
                "is_sink": "false"
              }
            },
            {
              "term": {
                "has_after_cover": "true"
              }
            },
            {
              "term": {
                "has_before_cover": "true"
              }
            },
            {
              "terms": {
                "content_level": [6, 5, 4, 3.5, 3]
              }
            },
            {
              "term": {
                "content_simi_bol_show": 0
              }
            }
          ],
          "must_not": [{ "term": { "show_by_index": 2 } }]
        }
      }
    }
  },
  "size": ${size},
  "_source": {"include": ${pretty(render(includes))}},
  "sort": [
    {
      "_script": {
        "type": "number",
        "script": {
          "id": "diary-recommend",
          "params": { "user_city_tag_id": ${cityId} }
        },
        "order": "desc"
      }
    },
    { "_score": { "order": "desc" } },
    { "has_video_cover": { "order": "asc" } },
    { "offline_score": { "order": "desc" } },
    { "good_click": { "order": "desc" } },
    { "last_update_time": { "order": "desc" } }
  ]
}
""".stripMargin

    // println("diary query:")
    // println(query)
    query
  }

  def generateTractateQuery(projects: List[String], secondDemands: List[String]): String = {
    val size = 30
    val includes = List("id", "tags_v3")

    val functions = new ListBuffer[JObject]()
    val shoulds = new ListBuffer[JObject]()
    if (projects.size > 0) {
      val promotes: List[JObject] =
        List(("terms" -> ("tags_v3" -> projects)), ("term" -> ("is_promote" -> true)))

      functions += ("filter" -> ("bool" -> ("must" -> promotes))) ~ ("weight" -> 60)

      functions += ("filter" -> ("terms" -> { "tags_v3" -> projects })) ~ ("weight" -> 50)

      shoulds += ("terms" -> ("tags_v3" -> projects))
    }

    if (secondDemands.size > 0) {
      functions += ("filter" -> ("terms" -> ("second_demands" -> secondDemands))) ~ ("weight" -> 40)

      shoulds += ("terms" -> ("second_demands" -> secondDemands))
    }

    val query = s"""
{
  "query": {
    "function_score": {
      "functions": ${pretty(render(functions.toList))},
      "boost_mode": "replace",
      "score_mode": "max",
      "query": {
        "bool": {
          "should": ${pretty(render(shoulds.toList))},
          "minimum_should_match": 1,
          "must": [
            {
              "term": {
                "is_online": "true"
              }
            },
            {
              "terms": {
                "content_level": [6, 5, 4, 3.5, 3]
              }
            }
          ],
          "must_not": [
            { "term": { "status": 4 } },
            { "term": { "show_by_index": 2 } }
          ]
        }
      }
    }
  },
  "size": ${size},
  "_source": {"include": ${pretty(render(includes))}}
}
""".stripMargin

    // println("tractate query:")
    // println(query)
    query
  }
}
